@@ -517,6 +517,10 @@ def MOL2read(filename, frame=None, topoloc=None, singlemol=True):
     molnum = 0
     with open(filename, "r") as f:
         for line in f:
+            line = line.strip()
+            if len(line) == 0:  # Ignore empty lines
+                continue
+
             if line.startswith("@<TRIPOS>MOLECULE"):
                 section = None
                 molnum += 1
@@ -735,8 +739,8 @@ def _getPDB(pdbid):
         # or the PDB website
         from moleculekit.rcsb import _getRCSBtext
 
-        logger.info("Attempting PDB query for {:s}".format(pdbid))
-        url = "https://files.rcsb.org/download/{}.pdb".format(pdbid)
+        logger.info(f"Attempting PDB query for {pdbid}")
+        url = f"https://files.rcsb.org/download/{pdbid}.pdb"
         text = _getRCSBtext(url)
         filepath = string_to_tempfile(text.decode("ascii"), "pdb")
         tempfile = True
@@ -912,12 +916,12 @@ def PDBread(
         )
     topodtypes = {
         "record": str,
-        "serial": np.int,
+        "serial": int,
         "name": str,
         "altloc": str,
         "resname": str,
         "chain": str,
-        "resid": np.int,
+        "resid": int,
         "insertion": str,
         "occupancy": np.float32,
         "beta": np.float32,
@@ -1180,7 +1184,7 @@ def PDBread(
     serials = parsedtopo.serial.values
     # if isinstance(serials[0], str) and np.any(serials == '*****'):
     #     logger.info('Non-integer serials were read. For safety we will discard all bond information and serials will be assigned automatically.')
-    #     topo.serial = np.arange(1, len(serials)+1, dtype=np.int)
+    #     topo.serial = np.arange(1, len(serials)+1, dtype=int)
     if np.max(parsedbonds.max()) > np.max(serials):
         logger.info(
             "Bond indexes in PDB file exceed atom indexes. For safety we will discard all bond information."
@@ -1559,7 +1563,7 @@ def XTCread(filename, frame=None, topoloc=None):
                     ct.c_char_p(filename.encode("ascii")), natoms_r, ct.c_int(f)
                 )
                 if not retval:
-                    raise IOError("XTC file {} possibly corrupt.".format(filename))
+                    raise RuntimeError(f"XTC file {filename} possibly corrupt.")
                 if coords is None:
                     coords = np.zeros((natoms_r[0], 3, nframes), dtype=np.float32)
                 fidx = 0
@@ -1583,12 +1587,13 @@ def XTCread(filename, frame=None, topoloc=None):
             lib["libc"].free(retval)
 
     if np.size(coords, 2) == 0:
-        raise NameError("Malformed XTC file. No frames read from: {}".format(filename))
+        raise RuntimeError(f"Malformed XTC file. No frames read from: {filename}")
     if np.size(coords, 0) == 0:
-        raise NameError("Malformed XTC file. No atoms read from: {}".format(filename))
+        raise RuntimeError(f"Malformed XTC file. No atoms read from: {filename}")
 
     coords *= 10.0  # Convert from nm to Angstrom
     box *= 10.0  # Convert from nm to Angstrom
+    time *= 1E3  # Convert from ps to fs. This seems to be ACEMD3 specific. GROMACS writes other units in time
     nframes = coords.shape[2]
     if len(step) != nframes or np.sum(step) == 0:
         step = np.arange(nframes)
@@ -2124,6 +2129,55 @@ def PREPIread(filename, frame=None, topoloc=None):
 
     return MolFactory.construct(topo, None, filename, frame)
 
+def SDFread(filename, frame=None, topoloc=None):
+    # Some (mostly correct) info here: www.nonlinear.com/progenesis/sdf-studio/v0.9/faq/sdf-file-format-guidance.aspx
+    chargemap = {"7": -3, "6": -2, "5": -1, "0": 0, "3": 1, "2": 2, "1": 3, "4": 0}
+
+    with open(filename, "r") as f:
+        lines = f.readlines()
+        molend = False
+        for line in lines:
+            if "V3000" in line:
+                raise RuntimeError("Moleculekit does not support parsing V3000 SDF files yet.")
+            if line.strip() == "$$$$":
+                molend = True
+            elif molend and line.strip() != "":
+                logger.warning("MoleculeKit will only read the first molecule from the SDF file.")
+
+        topo = Topology()
+        coords = []
+        mol_start = 0
+
+        n_atoms = int(lines[mol_start+3][:3])
+        n_bonds = int(lines[mol_start+3][3:6])
+
+        coord_start = mol_start + 4
+        bond_start = coord_start + n_atoms
+        bond_end = bond_start + n_bonds
+        for n in range(coord_start, bond_start):
+            pieces = lines[n].strip().split()
+            coords.append(list(map(float, pieces[:3])))
+            topo.element.append(pieces[3])
+            topo.name.append(pieces[3])
+            topo.serial.append(n-coord_start)
+            topo.charge.append(chargemap[pieces[5]])
+
+        for n in range(bond_start, bond_end):
+            idx1, idx2, bond_type = lines[n].strip().split()[:3]
+            topo.bonds.append([int(idx1)-1, int(idx2)-1])
+            topo.bondtype.append(bond_type)
+
+        for line in lines[bond_end:]:
+            if line.strip() == "$$$$":
+                break
+            if line.startswith("M  CHG"):  # These charges are more correct
+                pairs = line.strip().split()[3:]
+                for cc in range(0, len(pairs), 2):
+                    topo.charge[int(pairs[cc])-1] = int(pairs[cc+1])
+            
+    traj = Trajectory(coords=np.vstack(coords))
+    return MolFactory.construct(topo, traj, filename, frame)
+
 
 # Register here all readers with their extensions
 _TOPOLOGY_READERS = {
@@ -2142,6 +2196,7 @@ _TOPOLOGY_READERS = {
     "cif": PDBXMMCIFread,
     "rtf": RTFread,
     "prepi": PREPIread,
+    "sdf": SDFread,
 }
 
 _MDTRAJ_TOPOLOGY_EXTS = [
@@ -2237,6 +2292,31 @@ class _TestReaders(unittest.TestCase):
         mol = Molecule(os.path.join(testfolder, 'protein.mol2'))
         mol = Molecule(os.path.join(testfolder, 'ligand.mol2'))
 
+    def test_sdf(self):
+        sdf_file = os.path.join(self.testfolder(), 'benzamidine-3PTB-pH7.sdf')
+        ref_file = os.path.join(self.testfolder(), 'benzamidine-3PTB-pH7.npz')
+        ref_data = np.load(ref_file, allow_pickle=True)
+        mol = Molecule(sdf_file)
+        assert mol.numAtoms == 18
+        assert np.all(mol.name == mol.element)
+        assert np.array_equal(mol.element, ref_data["element"])
+        assert np.array_equal(mol.coords, ref_data["coords"])
+        assert np.array_equal(mol.charge, ref_data["charge"])
+        assert np.array_equal(mol.bonds, ref_data["bonds"])
+        assert np.array_equal(mol.bondtype, ref_data["bondtype"])
+
+        sdf_file = os.path.join(self.testfolder(), 'S98_ideal.sdf')
+        ref_file = os.path.join(self.testfolder(), 'S98_ideal.npz')
+        ref_data = np.load(ref_file, allow_pickle=True)
+        mol = Molecule(sdf_file)
+        assert mol.numAtoms == 34
+        assert np.all(mol.name == mol.element)
+        assert np.array_equal(mol.element, ref_data["element"])
+        assert np.array_equal(mol.coords, ref_data["coords"])
+        assert np.array_equal(mol.charge, ref_data["charge"])
+        assert np.array_equal(mol.bonds, ref_data["bonds"])
+        assert np.array_equal(mol.bondtype, ref_data["bondtype"])
+
     def test_gjf(self):
         testfolder = self.testfolder('3L5E')
         mol = Molecule(os.path.join(testfolder, 'ligand.gjf'))
@@ -2294,6 +2374,25 @@ class _TestReaders(unittest.TestCase):
         mol.read([os.path.join(self.testfolder('4RWS'), 'traj.xtc')], frames=[1])
         assert np.array_equal(tmpcoo[:, :, 1], np.squeeze(mol.coords)), 'Specific frame reading not working'
 
+    def test_acemd3_xtc_fstep(self):
+        from moleculekit.util import tempname
+
+        mol = Molecule(os.path.join(self.testfolder(), 'aladipep_traj_4fs_100ps.xtc'))
+        refstep = np.array([ 10,  20,  30,  40,  50,  60,  70,  80,  90, 100, 110, 120, 130, 140, 150, 160, 170, 180, 190, 200])
+        reftime = np.array([ 40.,  80., 120., 160., 200., 240., 280., 320., 360., 400., 440.,
+                            480., 520., 560., 600., 640., 680., 720., 760., 800.], dtype=np.float32)
+        assert np.array_equal(mol.step, refstep)
+        assert np.allclose(mol.time, reftime)
+        assert abs(mol.fstep - 4E-5) < 1E-12
+
+        # Test that XTC writing doesn't mess up the times
+        tmpfile = tempname(suffix=".xtc")
+        mol.write(tmpfile)
+
+        mol2 = Molecule(tmpfile)
+        assert mol.fstep == mol2.fstep
+        assert np.array_equal(mol.time, mol2.time)
+
     def test_gromacs_top(self):
         mol = Molecule(os.path.join(self.testfolder(), 'gromacs.top'))
         assert np.array_equal(mol.name, ['C1', 'O2', 'N3', 'H4', 'H5', 'N6', 'H7', 'H8'])
