@@ -13,27 +13,37 @@ import numpy as np
 from rdkit import Chem
 from rdkit.Chem.rdchem import HybridizationType, BondType
 from collections import defaultdict
-from moleculekit.smallmol.util import openbabelConvert, _depictMol, depictMultipleMols, convertToString
+from moleculekit.smallmol.util import _depictMol, depictMultipleMols, convertToString
+from moleculekit.tools.obabel_tools import openbabelConvert
 import logging
+
 logger = logging.getLogger(__name__)
 
 
 _bondtypes_IdxToType = BondType.values
-_bondtypes_TypeToString = defaultdict(lambda: 'un')
-_bondtypes_TypeToString.update({BondType.SINGLE: '1',
-                                BondType.DOUBLE: '2',
-                                BondType.TRIPLE: '3',
-                                BondType.AROMATIC: 'ar'})
-_bondtypes_FullStringToType = {'SINGLE': BondType.SINGLE,
-                           'DOUBLE': BondType.DOUBLE,
-                           'TRIPLE': BondType.TRIPLE}
+_bondtypes_TypeToString = defaultdict(lambda: "un")
+_bondtypes_TypeToString.update(
+    {
+        BondType.SINGLE: "1",
+        BondType.DOUBLE: "2",
+        BondType.TRIPLE: "3",
+        BondType.AROMATIC: "ar",
+    }
+)
+_bondtypes_FullStringToType = {
+    "SINGLE": BondType.SINGLE,
+    "DOUBLE": BondType.DOUBLE,
+    "TRIPLE": BondType.TRIPLE,
+}
 _bondtypes_StringToType = {val: key for key, val in _bondtypes_TypeToString.items()}
 
 _hybridizations_IdxToType = HybridizationType.values
-_hybridizations_StringToType = {'S': HybridizationType.S,
-                                'SP': HybridizationType.SP,
-                                'SP2': HybridizationType.SP2,
-                                'SP3': HybridizationType.SP3}
+_hybridizations_StringToType = {
+    "S": HybridizationType.S,
+    "SP": HybridizationType.SP,
+    "SP2": HybridizationType.SP2,
+    "SP3": HybridizationType.SP3,
+}
 
 
 class SmallMol(object):
@@ -76,28 +86,46 @@ class SmallMol(object):
        :attributes:
     """
 
-    _atom_fields = ['idx', 'name', 'charge', 'formalcharge', 'element',  'chiral', 'hybridization',
-                    'neighbors', 'bondtype', 'coords', 'chiraltags']
+    _atom_fields = [
+        "idx",
+        "name",
+        "charge",
+        "formalcharge",
+        "element",
+        "chiral",
+        "hybridization",
+        "neighbors",
+        "bondtype",
+        "coords",
+        "chiraltags",
+    ]
 
-    _mol_fields = ['ligname', '_mol']
+    _mol_fields = ["ligname", "_mol"]
 
-    _dtypes = {'idx': np.int,
-                    'name': object,
-                    'charge': np.float32,
-                    'formalcharge': np.int,
-                    'element': object,
-                    'chiral': object,
-                    'hybridization': int,
-                    'neighbors': object,
-                    'bondtype': object,
-                    'coords': np.float32,
-                    'chiraltags': object,
-                    'bonds': np.uint32
-                    }
+    _dtypes = {
+        "idx": int,
+        "name": object,
+        "charge": np.float32,
+        "formalcharge": int,
+        "element": object,
+        "chiral": object,
+        "hybridization": int,
+        "neighbors": object,
+        "bondtype": object,
+        "coords": np.float32,
+        "chiraltags": object,
+        "bonds": np.uint32,
+    }
 
-
-
-    def __init__(self, mol, ignore_errors=False, force_reading=False, fixHs=True, removeHs=False, verbose=True):
+    def __init__(
+        self,
+        mol,
+        ignore_errors=False,
+        force_reading=False,
+        fixHs=True,
+        removeHs=False,
+        verbose=True,
+    ):
         self._frame = 0
 
         self._mol = self._initializeMolObj(mol, force_reading, ignore_errors, verbose)
@@ -105,7 +133,6 @@ class SmallMol(object):
             self.removeHs()
         if fixHs:
             self.addHs(addCoords=True)
-        
 
     def _initializeMolObj(self, mol, force_reading, ignore_errors, verbose=True):
         """
@@ -150,10 +177,20 @@ class SmallMol(object):
                 # load pdb file
                 elif name_suffix == ".pdb":
                     _mol = Chem.MolFromPDBFile(mol, removeHs=False)
+                # load single-molecule sdf file
+                elif name_suffix == ".sdf":
+                    sms = Chem.SDMolSupplier(mol, removeHs=False)
+                    if len(sms) != 1:
+                        logger.warning(
+                            f"More than one molecules found in {mol}. SmallMol will only read the first. If you want to read them all use the SmallMolLib class."
+                        )
+                    _mol = sms[0]
                 # if the file failed to be loaded and 'force_reading' = True, file convert to sdf and than loaded
                 if _mol is None and force_reading:
-                    logger.warning('Reading {} with force_reading procedure'.format(mol))
-                    sdf = openbabelConvert(mol, name_suffix, 'sdf')
+                    logger.warning(
+                        "Reading {} with force_reading procedure".format(mol)
+                    )
+                    sdf = openbabelConvert(mol, name_suffix, "sdf")
                     _mol = Chem.SDMolSupplier(sdf, removeHs=False)[0]
                     os.remove(sdf)
             else:
@@ -167,95 +204,175 @@ class SmallMol(object):
 
         if _mol is None and not ignore_errors:
             if isinstance(mol, str):
-                frerr = ' Try by setting the force_reading option as True.' if not force_reading else ''
-                raise ValueError(f'Failed to read file {mol}.{frerr}')
+                frerr = (
+                    " Try by setting the force_reading option as True."
+                    if not force_reading
+                    else ""
+                )
+                raise ValueError(f"Failed to read file {mol}.{frerr}")
             elif isinstance(mol, Molecule):
-                raise ValueError('Failed converting Molecule to SmallMol')
+                raise ValueError("Failed converting Molecule to SmallMol")
             else:
                 raise RuntimeError(f"Failed reading molecule {mol}.")
 
         if natoms is not None and natoms != _mol.GetNumAtoms():
-            raise RuntimeError("Number of atoms changed while converting to rdkit molecule")
+            raise RuntimeError(
+                "Number of atoms changed while converting to rdkit molecule"
+            )
 
         return _mol
 
     @property
     def _idx(self):
-        return np.array([a.GetIdx() for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['idx'])
+        return np.array(
+            [a.GetIdx() for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes["idx"]
+        )
 
     @property
     def _element(self):
-        return np.array([a.GetSymbol() for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['element'])
+        return np.array(
+            [a.GetSymbol() for a in self._mol.GetAtoms()],
+            dtype=SmallMol._dtypes["element"],
+        )
 
     @property
     def _name(self):
-        return np.array(['{}{}'.format(a.GetSymbol(), a.GetIdx()) for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['name'])
+        return np.array(
+            ["{}{}".format(a.GetSymbol(), a.GetIdx()) for a in self._mol.GetAtoms()],
+            dtype=SmallMol._dtypes["name"],
+        )
 
     @property
     def _charge(self):
-        return np.array([a.GetPropsAsDict()['_TriposPartialCharge'] if a.HasProp('_TriposPartialCharge') else 0.0 for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['charge'])
+        return np.array(
+            [
+                a.GetPropsAsDict()["_TriposPartialCharge"]
+                if a.HasProp("_TriposPartialCharge")
+                else 0.0
+                for a in self._mol.GetAtoms()
+            ],
+            dtype=SmallMol._dtypes["charge"],
+        )
 
     @property
     def _formalcharge(self):
-        return np.array([a.GetFormalCharge() for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['formalcharge']) 
+        return np.array(
+            [a.GetFormalCharge() for a in self._mol.GetAtoms()],
+            dtype=SmallMol._dtypes["formalcharge"],
+        )
 
     @property
     def _chiral(self):
-        return np.array([a.GetPropsAsDict()['_CIPCode'] if a.HasProp('_CIPCode') else '' for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['chiral'])  
+        return np.array(
+            [
+                a.GetPropsAsDict()["_CIPCode"] if a.HasProp("_CIPCode") else ""
+                for a in self._mol.GetAtoms()
+            ],
+            dtype=SmallMol._dtypes["chiral"],
+        )
 
     @property
     def _hybridization(self):
-        return np.array([int(a.GetHybridization()) for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['hybridization'])
+        return np.array(
+            [int(a.GetHybridization()) for a in self._mol.GetAtoms()],
+            dtype=SmallMol._dtypes["hybridization"],
+        )
 
     @property
     def _neighbors(self):
-        return np.array([[na.GetIdx() for na in a.GetNeighbors()] for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['neighbors']) 
+        return np.array(
+            [[na.GetIdx() for na in a.GetNeighbors()] for a in self._mol.GetAtoms()],
+            dtype=SmallMol._dtypes["neighbors"],
+        )
 
     @property
     def _neighborsbondtype(self):
-        return np.array([[self._mol.GetBondBetweenAtoms(int(a), int(n)).GetBondType() for n in neighs] for a, neighs in enumerate(self._neighbors)], dtype=SmallMol._dtypes['bondtype'])
+        return np.array(
+            [
+                [
+                    self._mol.GetBondBetweenAtoms(int(a), int(n)).GetBondType()
+                    for n in neighs
+                ]
+                for a, neighs in enumerate(self._neighbors)
+            ],
+            dtype=SmallMol._dtypes["bondtype"],
+        )
 
     @property
     def _bonds(self):
-        return np.vstack([[bo.GetBeginAtomIdx(), bo.GetEndAtomIdx()] for bo in self._mol.GetBonds()]).astype(SmallMol._dtypes['bonds'])
+        bonds = [
+            [bo.GetBeginAtomIdx(), bo.GetEndAtomIdx()] for bo in self._mol.GetBonds()
+        ]
+        if len(bonds):
+            return np.vstack(bonds).astype(SmallMol._dtypes["bonds"])
+        else:
+            return np.zeros((0, 2), dtype=SmallMol._dtypes["bonds"])
 
     @property
     def _bondtype(self):
-        return np.array([_bondtypes_TypeToString[self._mol.GetBondBetweenAtoms(int(b1), int(b2)).GetBondType()] for b1, b2 in self._bonds], dtype=SmallMol._dtypes['bondtype'])
+        return np.array(
+            [
+                _bondtypes_TypeToString[
+                    self._mol.GetBondBetweenAtoms(int(b1), int(b2)).GetBondType()
+                ]
+                for b1, b2 in self._bonds
+            ],
+            dtype=SmallMol._dtypes["bondtype"],
+        )
 
     @property
     def _chiraltags(self):
-        return np.array([a.GetChiralTag() for a in self._mol.GetAtoms()], dtype=SmallMol._dtypes['chiraltags'])
+        return np.array(
+            [a.GetChiralTag() for a in self._mol.GetAtoms()],
+            dtype=SmallMol._dtypes["chiraltags"],
+        )
 
     @property
     def ligname(self):
-        return self._mol.GetProp('_Name') if self._mol.HasProp('_Name') else 'UNK'
+        return self._mol.GetProp("_Name") if self._mol.HasProp("_Name") else "UNK"
 
     @property
     def _resname(self):
-        return np.array([self.ligname.strip() for _ in range(self.numAtoms)], dtype=object)
+        return np.array(
+            [self.ligname.strip() for _ in range(self.numAtoms)], dtype=object
+        )
 
     @property
     def _coords(self):
         if self._mol.GetNumConformers() != 0:
-            return np.array(np.stack([cc.GetPositions() for cc in self._mol.GetConformers()], axis=2), dtype=SmallMol._dtypes['coords'])
+            return np.array(
+                np.stack(
+                    [cc.GetPositions() for cc in self._mol.GetConformers()], axis=2
+                ),
+                dtype=SmallMol._dtypes["coords"],
+            )
         else:
-            return np.zeros((self.numAtoms, 3, 0), dtype=SmallMol._dtypes['coords'])
+            return np.zeros((self.numAtoms, 3, 0), dtype=SmallMol._dtypes["coords"])
 
     @property
     def _atomtype(self):
-        return np.array([a.GetPropsAsDict()['_TriposAtomType'] if a.HasProp('_TriposAtomType') else '' for a in self._mol.GetAtoms()], dtype=object)
+        return np.array(
+            [
+                a.GetPropsAsDict()["_TriposAtomType"]
+                if a.HasProp("_TriposAtomType")
+                else ""
+                for a in self._mol.GetAtoms()
+            ],
+            dtype=object,
+        )
 
     @property
     def frame(self):
         if self._frame < 0 or self._frame >= self.numFrames:
-            raise NameError("frame out of range")
+            raise RuntimeError("frame out of range")
         return self._frame
 
     @frame.setter
     def frame(self, value):
         if value < 0 or ((self.numFrames != 0) and (value >= self.numFrames)):
-            raise NameError("Frame index out of range. Molecule contains {} frame(s). Frames are 0-indexed.".format(self.numFrames))
+            raise RuntimeError(
+                f"Frame index out of range. Molecule contains {self.numFrames} frame(s). Frames are 0-indexed."
+            )
         self._frame = value
 
     # Dummy properties for compatibility with Molecule
@@ -281,23 +398,23 @@ class SmallMol(object):
 
     @property
     def _insertion(self):
-        return np.array(['' for _ in range(self.numAtoms)], dtype=object)
+        return np.array(["" for _ in range(self.numAtoms)], dtype=object)
 
     @property
     def _chain(self):
-        return np.array(['' for _ in range(self.numAtoms)], dtype=object)
+        return np.array(["" for _ in range(self.numAtoms)], dtype=object)
 
     @property
     def _segid(self):
-        return np.array(['' for _ in range(self.numAtoms)], dtype=object)
+        return np.array(["" for _ in range(self.numAtoms)], dtype=object)
 
     @property
     def _altloc(self):
-        return np.array(['' for _ in range(self.numAtoms)], dtype=object)
+        return np.array(["" for _ in range(self.numAtoms)], dtype=object)
 
     @property
     def _record(self):
-        return np.array(['HETATM' for _ in range(self.numAtoms)], dtype=object)
+        return np.array(["HETATM" for _ in range(self.numAtoms)], dtype=object)
 
     @property
     def numAtoms(self):
@@ -316,7 +433,7 @@ class SmallMol(object):
 
     def filter(self, sel):
         # Not implemented
-        raise NotImplementedError('Filtering atoms not supported yet.')
+        raise NotImplementedError("Filtering atoms not supported yet.")
 
     def copy(self):
         """
@@ -327,9 +444,11 @@ class SmallMol(object):
         newsmallmol : :class:`SmallMol`
             A copy of the object
         """
-        return SmallMol(Chem.Mol(self._mol))  # Chem.Mol creates a deep copy of the C++ object
+        return SmallMol(
+            Chem.Mol(self._mol), fixHs=False, removeHs=False
+        )  # Chem.Mol creates a deep copy of the C++ object
 
-    def get(self, returnField, sel='all', convertType=True, invert=False):
+    def get(self, returnField, sel="all", convertType=True, invert=False):
         """
         Returns the property for the atom specified with the selection. The selection is another atom property
 
@@ -374,25 +493,25 @@ class SmallMol(object):
         array([<rdkit.Chem.rdchem.Atom object at 0x7faf616dd120>,
                <rdkit.Chem.rdchem.Atom object at 0x7faf616dd170>], dtype=object)
         """
-        if sel == 'all':
-            sel = 'idx {}'.format(convertToString(self._idx.tolist()))
+        if sel == "all":
+            sel = "idx {}".format(convertToString(self._idx.tolist()))
         # get the field key and the value to grep
         key = sel.split()[0]
         selector = sel.split()[1:]
 
         if key not in self._atom_fields:
-            raise KeyError('The property passed {} does not exist'.format(key))
+            raise KeyError("The property passed {} does not exist".format(key))
         if len(selector) == 0:
-            raise ValueError('No selection was provided')
+            raise ValueError("No selection was provided")
 
         # get the returnField and process exceptional field
-        if hasattr(self, '_'+key):
-            _arrayFrom = self.__getattribute__('_'+key)
+        if hasattr(self, "_" + key):
+            _arrayFrom = self.__getattribute__("_" + key)
         else:
             _arrayFrom = self.__getattribute__(key)
 
         # special selector for hybridization: can be idx, or rdkit.Chem.rdchem.HybridizationType
-        if key == 'hybridization':
+        if key == "hybridization":
             try:
                 selector = [_hybridizations_StringToType[s.upper()] for s in selector]
             except:
@@ -406,17 +525,26 @@ class SmallMol(object):
             idxs = np.array([i for i in self._idx if i not in idxs])
         idxs = np.sort(idxs)
 
-        if returnField == 'atomobject':
+        if returnField == "atomobject":
             return self.getAtoms()[idxs]
-        elif returnField == 'bondtype':
+        elif returnField == "bondtype":
             if convertType:
-                return np.array([[_bondtypes_IdxToType[bt] for bt in neighbt] for neighbt in self._neighborsbondtype[idxs]], dtype=object)
+                return np.array(
+                    [
+                        [_bondtypes_IdxToType[bt] for bt in neighbt]
+                        for neighbt in self._neighborsbondtype[idxs]
+                    ],
+                    dtype=object,
+                )
             else:
                 return self._neighborsbondtype[idxs]
-        elif returnField == 'hybridization' and convertType:
-            _arrayTo = np.array([_hybridizations_IdxToType[v] for v in self._hybridization], dtype=object)
+        elif returnField == "hybridization" and convertType:
+            _arrayTo = np.array(
+                [_hybridizations_IdxToType[v] for v in self._hybridization],
+                dtype=object,
+            )
         else:
-            _arrayTo = self.__getattribute__('_'+returnField)
+            _arrayTo = self.__getattribute__("_" + returnField)
 
         return _arrayTo[idxs]
 
@@ -448,13 +576,15 @@ class SmallMol(object):
         else:
             _btype = bondtype
 
-        atomIdx_sel1 = self.get('idx', sel1)
-        neighbors_sel1 = self.get('neighbors', sel1)
-        bondtypes_sel1 = self.get('bondtype', sel1)
-        atomIdx_sel2 = self.get('idx', sel2)
+        atomIdx_sel1 = self.get("idx", sel1)
+        neighbors_sel1 = self.get("neighbors", sel1)
+        bondtypes_sel1 = self.get("bondtype", sel1)
+        atomIdx_sel2 = self.get("idx", sel2)
 
         founds = []
-        for aIdx, neighbors_set, btype_set in zip(atomIdx_sel1, neighbors_sel1, bondtypes_sel1):
+        for aIdx, neighbors_set, btype_set in zip(
+            atomIdx_sel1, neighbors_sel1, bondtypes_sel1
+        ):
             for neighbor, btype in zip(neighbors_set, btype_set):
                 if neighbor in atomIdx_sel2:
                     btype_str = _bondtypes_TypeToString[btype]
@@ -495,14 +625,14 @@ class SmallMol(object):
 
         _chirals = self._chiral
 
-        idxs = np.where(_chirals != '')[0]
+        idxs = np.where(_chirals != "")[0]
         if len(idxs) == 0:
             return False
         if returnDetails:
             idxs = idxs.astype(str)
-            idxs_str = ' '.join(idxs)
-            names = self.get('name', 'idx {}'.format(idxs_str))
-            chirals = self.get('chiral', 'idx {}'.format(idxs_str))
+            idxs_str = " ".join(idxs)
+            names = self.get("name", "idx {}".format(idxs_str))
+            chirals = self.get("chiral", "idx {}".format(idxs_str))
 
             return True, [(a, c) for a, c in zip(names, chirals)]
 
@@ -521,7 +651,9 @@ class SmallMol(object):
         coords = self._coords[:, :, self.frame]
         return coords.mean(axis=0).astype(np.float32)
 
-    def generateConformers(self, num_confs=400,  optimizemode='mmff', align=True, append=True):
+    def generateConformers(
+        self, num_confs=400, optimizemode="mmff", align=True, append=True
+    ):
         """
         Generates ligand conformers
 
@@ -540,7 +672,11 @@ class SmallMol(object):
             Default: True
 
         """
-        from rdkit.Chem.AllChem import UFFOptimizeMolecule, MMFFOptimizeMolecule, EmbedMultipleConfs
+        from rdkit.Chem.AllChem import (
+            UFFOptimizeMolecule,
+            MMFFOptimizeMolecule,
+            EmbedMultipleConfs,
+        )
         from rdkit.Chem.rdMolAlign import AlignMolConformers
 
         if not append:
@@ -552,14 +688,20 @@ class SmallMol(object):
         mol = Chem.AddHs(mol)
 
         # generating conformations
-        ids = EmbedMultipleConfs(mol, clearConfs=False, numConfs=num_confs, pruneRmsThresh=1., maxAttempts=10000)
-        if optimizemode not in ['uff', 'mmff']:
+        ids = EmbedMultipleConfs(
+            mol,
+            clearConfs=False,
+            numConfs=num_confs,
+            pruneRmsThresh=1.0,
+            maxAttempts=10000,
+        )
+        if optimizemode not in ["uff", "mmff"]:
             raise ValueError('Unknown optimizemode. Should be  "uff", "mmff"')
         # optimizing conformations depends on the optimizemode passed
         for id in ids:
-            if optimizemode == 'mmff':
+            if optimizemode == "mmff":
                 MMFFOptimizeMolecule(mol, confId=id)
-            elif optimizemode == 'uff':
+            elif optimizemode == "uff":
                 UFFOptimizeMolecule(mol, confId=id)
 
         if align:
@@ -578,39 +720,47 @@ class SmallMol(object):
 
         pyO3A = GetO3A(self._mol, refmol)
         rmsd = pyO3A.Align()
-        logger.info('Alignment with a RMSD of {}'.format(rmsd))
+        logger.info("Alignment with a RMSD of {}".format(rmsd))
 
-    def dropFrames(self, frames='all'):
+    def dropFrames(self, frames="all"):
         if isinstance(frames, int):
-            frames = np.array([frames,])
+            frames = np.array(
+                [
+                    frames,
+                ]
+            )
         if isinstance(frames, list):
             frames = np.array(frames)
-        if frames == 'all':
+        if frames == "all":
             ids = [cc.GetId() for cc in self._mol.GetConformers()]
             for f in ids:
                 self._mol.RemoveConformer(int(f))
         else:
             if np.any(frames >= self.numFrames):
-                raise RuntimeError('Cannot remove more frames than existing which is {}'.format(self.numFrames))
+                raise RuntimeError(
+                    "Cannot remove more frames than existing which is {}".format(
+                        self.numFrames
+                    )
+                )
             ids = [cc.GetId() for cc in self._mol.GetConformers()]
             for f in frames:
                 self._mol.RemoveConformer(int(ids[f]))
 
     def write(self, fname, frames=None, merge=True):
         ext = os.path.splitext(fname)[-1]
-        if ext == '.sdf':
+        if ext == ".sdf":
             chemwrite = Chem.SDWriter
             # If merge is set as True a unique file is generated
             if merge:
                 writer = chemwrite(fname)
                 writer.write(self._mol)
-            else: # If merge is set as False a file is created for each conformer
+            else:  # If merge is set as False a file is created for each conformer
                 if frames is None:
                     frames = list(range(self.numFrames))
                 os.makedirs(os.path.dirname(fname), exist_ok=True)
                 for i in frames:
                     fname_first = os.path.splitext(fname)[0]
-                    currfname = os.path.join('{}_{}{}'.format(fname_first, i, ext))
+                    currfname = os.path.join("{}_{}{}".format(fname_first, i, ext))
                     writer = chemwrite(currfname)
                     writer.write(self._mol, confId=i)
         else:
@@ -635,8 +785,9 @@ class SmallMol(object):
             The smarts string
         """
         from copy import deepcopy
+
         rmol = deepcopy(self._mol)
-        if not explicitHs and len(np.where(self._element == 'H')[0]) != 0:
+        if not explicitHs and len(np.where(self._element == "H")[0]) != 0:
             rmol = Chem.RemoveHs(rmol)
 
         return Chem.MolToSmarts(rmol, isomericSmiles=True)
@@ -658,9 +809,10 @@ class SmallMol(object):
             The smiles string
         """
         from copy import deepcopy
+
         rmol = deepcopy(self._mol)
 
-        if not explicitHs and len(np.where(self._element == 'H')[0]) != 0:
+        if not explicitHs and len(np.where(self._element == "H")[0]) != 0:
             rmol = Chem.RemoveHs(rmol)
 
         if kekulizeSmile:
@@ -688,12 +840,12 @@ class SmallMol(object):
 
         from tempfile import NamedTemporaryFile
 
-        if np.all(mol.bondtype == 'un') and np.all(mol.atomtype == ""):
-            tmppdb = NamedTemporaryFile(suffix='.pdb').name
+        if np.all(mol.bondtype == "un") and np.all(mol.atomtype == ""):
+            tmppdb = NamedTemporaryFile(suffix=".pdb").name
             mol.write(tmppdb)
             return tmppdb
         else:
-            tmpmol2 = NamedTemporaryFile(suffix='.mol2').name
+            tmpmol2 = NamedTemporaryFile(suffix=".mol2").name
             mol.write(tmpmol2)
             return tmpmol2
 
@@ -724,13 +876,15 @@ class SmallMol(object):
             ids = np.arange(self.numFrames)
 
         if self.numFrames == 0:
-            raise NoConformerError("No Conformers are found in the molecule. Generate at least one conformer.")
+            raise NoConformerError(
+                "No Conformers are found in the molecule. Generate at least one conformer."
+            )
         elif not isinstance(ids, list) and not isinstance(ids, np.ndarray):
-            raise ValueError('The argument ids should be a list of confomer ids')
+            raise ValueError("The argument ids should be a list of confomer ids")
 
         mol = Molecule()
         mol.empty(self.numAtoms)
-        mol.record[:] = 'HETATM'
+        mol.record[:] = "HETATM"
         mol.resname[:] = self.ligname[:3]
         mol.resid[:] = self._resid
         mol.coords = self._coords[:, :, ids]
@@ -747,8 +901,18 @@ class SmallMol(object):
         mol.atomtype = self._atomtype
         return mol
 
-    def depict(self, sketch=True, filename=None, ipython=False, optimize=False, optimizemode='std', removeHs=True,
-               atomlabels=None, highlightAtoms=None, resolution=(400, 200)):
+    def depict(
+        self,
+        sketch=True,
+        filename=None,
+        ipython=False,
+        optimize=False,
+        optimizemode="std",
+        removeHs=True,
+        atomlabels=None,
+        highlightAtoms=None,
+        resolution=(400, 200),
+    ):
         """
         Depicts the molecules. It is possible to save it into an svg file and also generates a jupiter-notebook rendering
 
@@ -788,14 +952,25 @@ class SmallMol(object):
         >>> sm.depict(ipython=True, sketch=True,highlightAtoms=ids.tolist(), removeHs=False)  # doctest: +SKIP
         """
         from rdkit import Chem
-        from rdkit.Chem.AllChem import Compute2DCoords, EmbedMolecule, MMFFOptimizeMolecule, ETKDG
+        from rdkit.Chem.AllChem import (
+            Compute2DCoords,
+            EmbedMolecule,
+            MMFFOptimizeMolecule,
+            ETKDG,
+        )
         from copy import deepcopy
 
         if sketch and optimize:
-            raise ValueError('Impossible to use optmization in  2D sketch representation')
+            raise ValueError(
+                "Impossible to use optimization in  2D sketch representation"
+            )
 
-        if optimizemode not in ['std', 'mmff']:
-            raise ValueError('Optimization mode {} not understood. Can be "std" or "ff"'.format(optimizemode))
+        if optimizemode not in ["std", "mmff"]:
+            raise ValueError(
+                'Optimization mode {} not understood. Can be "std" or "ff"'.format(
+                    optimizemode
+                )
+            )
 
         elements = self._element
         indexes = self._idx
@@ -804,37 +979,46 @@ class SmallMol(object):
 
         _mol = deepcopy(self._mol)
 
-
         if sketch:
             Compute2DCoords(_mol)
 
         if removeHs:
             _mol = Chem.RemoveHs(_mol)
-            elements = self.get('element', 'element H', invert=True)
-            indexes = self.get('idx', 'element H', invert=True)
-            formalcharges = self.get('formalcharge', 'element H', invert=True)
-            chirals = self.get('chiral', 'element H', invert=True)
+            elements = self.get("element", "element H", invert=True)
+            indexes = self.get("idx", "element H", invert=True)
+            formalcharges = self.get("formalcharge", "element H", invert=True)
+            chirals = self.get("chiral", "element H", invert=True)
 
-        _labelsFunc = ['a', 'i', 'c', '*']
+        _labelsFunc = ["a", "i", "c", "*"]
 
         if atomlabels is not None:
-            labels = atomlabels.split('%')[1:]
-            formalcharges = ['' if c == 0 else "+" if c == 1 else "-" for c in formalcharges]
-            chirals = ['' if c == '' else '*' for c in chirals]
+            labels = atomlabels.split("%")[1:]
+            formalcharges = [
+                "" if c == 0 else "+" if c == 1 else "-" for c in formalcharges
+            ]
+            chirals = ["" if c == "" else "*" for c in chirals]
             values = [elements, indexes, formalcharges, chirals]
 
             idxs = [_labelsFunc.index(l) for l in labels]
             labels_required = [values[i] for i in idxs]
-            atomlabels = ["".join([str(i) for i in a]) for a in list(zip(*labels_required))]
+            atomlabels = [
+                "".join([str(i) for i in a]) for a in list(zip(*labels_required))
+            ]
 
         if optimize:
-            if optimizemode == 'std':
+            if optimizemode == "std":
                 EmbedMolecule(_mol, ETKDG())
-            elif optimizemode == 'mmff':
+            elif optimizemode == "mmff":
                 MMFFOptimizeMolecule(_mol)
 
-        return _depictMol(_mol, filename=filename, ipython=ipython,  atomlabels=atomlabels,
-                          highlightAtoms=highlightAtoms, resolution=resolution)
+        return _depictMol(
+            _mol,
+            filename=filename,
+            ipython=ipython,
+            atomlabels=atomlabels,
+            highlightAtoms=highlightAtoms,
+            resolution=resolution,
+        )
 
     def addHs(self, addCoords=True):
         self._mol = Chem.AddHs(self._mol, addCoords=addCoords)
@@ -843,41 +1027,49 @@ class SmallMol(object):
         self._mol = Chem.RemoveHs(self._mol)
 
     def __repr__(self):
-        return '<{}.{} object at {}>\n'.format(self.__class__.__module__, self.__class__.__name__, hex(id(self))) \
-               + self.__str__()
+        return (
+            "<{}.{} object at {}>\n".format(
+                self.__class__.__module__, self.__class__.__name__, hex(id(self))
+            )
+            + self.__str__()
+        )
 
     def __str__(self):
         def formatstr(name, field):
             if isinstance(field, np.ndarray) or isinstance(field, list):
-                rep = '{} shape: {}'.format(name, np.shape(field))
-            elif field == 'reps':
-                rep = '{}: {}'.format(name, len(self.reps.replist))
+                rep = "{} shape: {}".format(name, np.shape(field))
+            elif field == "reps":
+                rep = "{}: {}".format(name, len(self.reps.replist))
             else:
-                rep = '{}: {}'.format(name, field)
+                rep = "{}: {}".format(name, field)
             return rep
 
-        rep = 'SmallMol with {} atoms and {} conformers'.format(self.numAtoms, self.numFrames)
+        rep = "SmallMol with {} atoms and {} conformers".format(
+            self.numAtoms, self.numFrames
+        )
         for p in sorted(self._atom_fields):
-            if p.startswith('_'):
+            if p.startswith("_"):
                 continue
-            rep += '\n'
-            rep += 'Atom field - {}'.format(p)
+            rep += "\n"
+            rep += "Atom field - {}".format(p)
         for j in sorted(self.__dict__.keys() - list(SmallMol._atom_fields)):
-            if j[0] == '_':
+            if j[0] == "_":
                 continue
-            rep += '\n'
+            rep += "\n"
             rep += formatstr(j, self.__dict__[j])
 
         return rep
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
 
     import doctest
     import os
     from moleculekit.home import home
     from moleculekit.smallmol.smallmollib import SmallMolLib
 
-    lib = SmallMolLib(os.path.join(home(dataDir='test-smallmol'), 'fda_drugs_light.sdf'))
-    sm = SmallMol(os.path.join(home(dataDir='test-smallmol'), 'benzamidine.mol2'))
-    doctest.testmod(extraglobs={'lib': lib.copy(), 'sm': sm.copy()})
-
+    lib = SmallMolLib(
+        os.path.join(home(dataDir="test-smallmol"), "fda_drugs_light.sdf")
+    )
+    sm = SmallMol(os.path.join(home(dataDir="test-smallmol"), "benzamidine.mol2"))
+    doctest.testmod(extraglobs={"lib": lib.copy(), "sm": sm.copy()})
